@@ -49,6 +49,10 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
+#ifdef MAC_INTEGRATION
+#  include <ige-mac-integration.h>
+#endif
+
 #define GLADE_CONFIG_FILENAME "glade-3.conf"
 
 #define GLADE_APP_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GLADE_TYPE_APP, GladeAppPrivate))
@@ -56,6 +60,7 @@
 enum
 {
 	UPDATE_UI,
+	SIGNAL_EDITOR_CREATED,
 	LAST_SIGNAL
 };
 
@@ -288,6 +293,12 @@ glade_app_update_ui_default (GladeApp *app)
 			glade_app_refresh_undo_redo_button (app, list->data, FALSE);
 }
 
+static void
+glade_app_signal_editor_created_default (GladeApp *app, GladeSignalEditor *signal_editor)
+{
+	glade_signal_editor_construct_signals_list (signal_editor);
+}
+
 static gboolean
 clipboard_view_on_delete_cb (GtkWidget *clipboard_view, GdkEvent *e, GladeApp *app)
 {
@@ -355,14 +366,22 @@ glade_app_get_locale_dir (void)
 static void
 build_package_paths (void)
 {
-#ifdef G_OS_WIN32
+#if defined (MAC_INTEGRATION) || defined (G_OS_WIN32)
 	gchar *prefix;
-	
+
+# ifdef G_OS_WIN32
 	prefix = g_win32_get_package_installation_directory_of_module (NULL);
+# else /* MAC_INTEGRATION */	
+	IgeMacBundle *bundle = ige_mac_bundle_get_default ();
+
+	prefix = g_build_filename (ige_mac_bundle_get_path (bundle), "Contents", "Resources", NULL);
+# endif 
+
 	pixmaps_dir  = g_build_filename (prefix, "share", PACKAGE, "pixmaps", NULL);
 	catalogs_dir = g_build_filename (prefix, "share", PACKAGE, "catalogs", NULL);
 	modules_dir  = g_build_filename (prefix, "lib", PACKAGE, "modules", NULL);
 	locale_dir   = g_build_filename (prefix, "share", "locale", NULL);
+
 	g_free (prefix);
 #else
 	catalogs_dir = g_strdup (GLADE_CATALOGSDIR);
@@ -382,6 +401,9 @@ glade_init_check (void)
 	if (initialised)
 		return;
 
+	/* Make sure path accessors work on osx */
+	g_type_init ();
+
 	build_package_paths ();
 
 	bindtextdomain (GETTEXT_PACKAGE, locale_dir);
@@ -396,6 +418,8 @@ glade_app_init (GladeApp *app)
 	static gboolean initialized = FALSE;
 	
 	app->priv = GLADE_APP_GET_PRIVATE (app);	
+
+	singleton_app = app;
 	
 	glade_init_check ();	
 	
@@ -453,6 +477,7 @@ glade_app_class_init (GladeAppClass * klass)
 	object_class->set_property = glade_app_set_property;
 
 	klass->update_ui_signal = glade_app_update_ui_default;
+	klass->signal_editor_created = glade_app_signal_editor_created_default;
 	klass->show_properties  = NULL;
 	klass->hide_properties  = NULL;
 
@@ -472,6 +497,26 @@ glade_app_class_init (GladeAppClass * klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
 
+	/**
+	 * GladeApp::signal-editor-created:
+	 * @gladeapp: the #GladeApp which received the signal.
+	 * @signal_editor: the new #GladeSignalEditor.
+	 *
+	 * Emitted when a new signal editor created.
+	 * A tree view is created in the default handler.
+	 * Connect your handler before the default handler for setting a custom column or renderer
+	 * and after it for connecting to the tree view signals
+	 */
+	glade_app_signals[SIGNAL_EDITOR_CREATED] =
+		g_signal_new ("signal-editor-created",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GladeAppClass,
+					       signal_editor_created),
+			      NULL, NULL,
+			      glade_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 1, G_TYPE_OBJECT);
+
 	g_object_class_install_property 
 		(object_class, PROP_ACTIVE_PROJECT,
 		 g_param_spec_object 
