@@ -25,10 +25,7 @@
 
 #include "config.h"
 
-#include <gtk/gtkalignment.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkimage.h>
+#include <gtk/gtk.h>
 
 #include "glade.h"
 #include "glade-palette-item.h"
@@ -340,7 +337,13 @@ static gboolean
 glade_palette_item_button_press (GtkWidget      *widget,
 				 GdkEventButton *event)
 {
-	if (event->type == GDK_BUTTON_PRESS && event->button == 3)
+	/* Give some kind of access in case of missing right button */
+	if (!event->type == GDK_BUTTON_PRESS &&
+	    (event->button == 3 || 
+	     (event->button == 1 && 
+	      ((event->state & GDK_MOD1_MASK) != 0 ||
+	       (event->state & GDK_MOD2_MASK) != 0 ||
+	       (event->state & GDK_MOD2_MASK) != 0))))
 	{
 		GladePaletteItemPrivate *priv = GLADE_PALETTE_ITEM_GET_PRIVATE (widget);
 
